'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var xml = _interopDefault(require('xml'));

function formatTime(timeInSeconds) {
    const hours = Math.floor(timeInSeconds / (60 * 60));
    timeInSeconds -= hours * 60 * 60;
    const minutes = Math.floor(timeInSeconds / 60);
    timeInSeconds -= minutes * 60;
    // left pad number with 0
    const leftPad = (num) => `${num}`.padStart(2, '0');
    const str = (hours ? `${leftPad(hours)}:` : '') +
        // (minutes ? `${leftPad(minutes)}:` : '00') +
        `${leftPad(minutes)}:` +
        leftPad(Math.round(timeInSeconds));
    return str;
}

const DOCTYPE = '<?xml version="1.0" encoding="utf-8"?>\n';
const CDATA = (foo) => ({
    _cdata: foo
});
var renderRSS = (ins) => {
    const { options, IToptions } = ins;
    let isAtom = false;
    let isContent = false;
    const makeITunesField = (field, content, text, href
    // child?: ITField,
    ) => {
        if (!(content || text || href))
            return undefined;
        const record = { ['itunes:' + field]: content };
        record._attr = { href, text };
        return record;
    };
    const categories = IToptions.categories.map((category) => {
        let finalItem;
        if (category.child) {
            finalItem = [
                {
                    _attr: {
                        text: category.cat
                    }
                },
                {
                    'itunes:category': {
                        _attr: {
                            text: category.child
                        }
                    }
                }
            ];
        }
        else {
            finalItem = {
                _attr: {
                    text: category.cat
                }
            };
        }
        return {
            'itunes:category': finalItem
        };
    });
    // console.log({ categories })
    const ITchannel = [
        makeITunesField('summary', IToptions.summary),
        makeITunesField('author', IToptions.author),
        makeITunesField('keywords', IToptions.keywords.join(',')),
        ...categories,
        { 'itunes:image': { _attr: { href: IToptions.image } } },
        makeITunesField('explicit', IToptions.explicit ? 'yes' : 'clean'),
        {
            'itunes:owner': [
                makeITunesField('name', CDATA(IToptions.owner.name)),
                makeITunesField('email', IToptions.owner.email)
            ]
        },
        // subtitle
        makeITunesField('type', IToptions.type)
    ];
    const channel = [
        // // SWYX: disabled for duplication
        // {
        //   'atom:link': {
        //     _attr: {
        //       href: invariant(
        //         options.feedLinks,
        //         'rss',
        //         'missing in your feed channel config options',
        //       ),
        //       rel: 'self',
        //       type: 'application/rss+xml',
        //     },
        //   },
        // },
        { title: options.title },
        { link: options.link },
        { language: 'en' },
        { description: options.description },
        { managingEditor: `${options.author.email} (${options.author.email})` },
        {
            pubDate: options.updated // TODO: use actual last pub date
                ? options.updated.toUTCString()
                : new Date().toUTCString()
        },
        ,
        {
            lastBuildDate: options.updated
                ? options.updated.toUTCString()
                : new Date().toUTCString()
        },
        { docs: options.link },
        {
            generator: options.generator ||
                'https://github.com/sw-yx/react-static-typescript-starter'
        },
        ...ITchannel
    ];
    const rss = [{ _attr: { version: '2.0' } }, { channel }];
    /**
     * Channel Image
     * http://cyber.law.harvard.edu/rss/rss.html#ltimagegtSubelementOfLtchannelgt
     */
    if (options.image) {
        channel.push({
            image: [
                { title: options.title },
                { url: options.image },
                { link: options.link }
            ]
        });
    }
    /**
     * Channel Copyright
     * http://cyber.law.harvard.edu/rss/rss.html#optionalChannelElements
     */
    if (options.copyright) {
        channel.push({ copyright: options.copyright });
    }
    /**
     * Channel Categories
     * http://cyber.law.harvard.edu/rss/rss.html#comments
     */
    ins.categories.forEach(category => {
        channel.push({ category });
    });
    /**
     * Feed URL
     * http://validator.w3.org/feed/docs/warning/MissingAtomSelfLink.html
     */
    const atomLink = options.feed || (options.feedLinks && options.feedLinks.atom);
    if (atomLink) {
        isAtom = true;
        channel.push({
            'atom:link': {
                _attr: {
                    href: atomLink,
                    rel: 'self',
                    type: 'application/rss+xml'
                }
            }
        });
    }
    /**
     * Hub for PubSubHubbub
     * https://code.google.com/p/pubsubhubbub/
     */
    if (options.hub) {
        isAtom = true;
        channel.push({
            'atom:link': {
                _attr: {
                    href: options.hub,
                    rel: 'hub'
                }
            }
        });
    }
    /**
     * Channel Categories
     * http://cyber.law.harvard.edu/rss/rss.html#hrelementsOfLtitemgt
     */
    ins.items.forEach((entry) => {
        let item = [];
        if (entry.title) {
            item.push({ title: CDATA(entry.title) });
        }
        if (entry.link) {
            item.push({ link: CDATA(entry.link) });
            // item.push({ link: entry.link })
        }
        if (entry.guid) {
            item.push({ guid: entry.guid });
        }
        else if (entry.link) {
            item.push({ guid: entry.link });
        }
        if (entry.date) {
            item.push({ pubDate: entry.date.toUTCString() });
        }
        if (entry.description) {
            item.push({ description: CDATA(entry.description) });
        }
        if (entry.content) {
            isContent = true;
            item.push({ 'content:encoded': CDATA(entry.content) });
        }
        /**
         * Item Author
         * http://cyber.law.harvard.edu/rss/rss.html#ltauthorgtSubelementOfLtitemgt
         */
        if (Array.isArray(entry.author)) {
            entry.author.map((author) => {
                if (author.email && author.name) {
                    item.push({ author: author.email + ' (' + author.name + ')' });
                }
            });
        }
        if (entry.image) {
            item.push({ enclosure: [{ _attr: { url: entry.image } }] });
        }
        const { itunes } = entry;
        if (itunes) {
            // // SWYX: TEMPORARY DISABLE BECAUSE DUPLICATE
            // item.push(
            //   makeITunesField(
            //     'image',
            //     '',
            //     undefined,
            //     itunes.image || IToptions.image,
            //   ),
            // )
            // // SWYX: TEMPORARY DISABLE BECAUSE DUPLICATE
            // item.push({
            //   guid: {
            //     _attr: {
            //       isPermaLink: 'false',
            //     },
            //     _cdata: invariant(itunes, 'mp3URL'),
            //   },
            // })
            item.push({
                enclosure: {
                    _attr: {
                        length: invariant(itunes, 'enclosureLength'),
                        type: 'audio/mpeg',
                        url: invariant(itunes, 'mp3URL')
                    }
                }
            });
            // item.push(
            //   makeITunesField('image', '', undefined, itunes.image || options.image),
            // )
            item.push(makeITunesField('duration', formatTime(itunes.duration)));
            item.push(makeITunesField('explicit', itunes.explicit ? 'yes' : 'no'));
            if (itunes.keywords && itunes.keywords.length)
                item.push(makeITunesField('keywords', itunes.keywords.join(',')));
            item.push(makeITunesField('subtitle', CDATA(invariant(itunes, 'subtitle'))));
            item.push(makeITunesField('episodeType', itunes.episodeType));
            if (itunes.episode)
                item.push(makeITunesField('episode', itunes.episode));
            if (itunes.season)
                item.push(makeITunesField('season', itunes.season));
            //  // SWYX: temporary disable for duplication
            // item.push({ 'content:encoded': CDATA(itunes.contentEncoded) })
        }
        channel.push({ item });
    });
    if (isContent) {
        rss[0]._attr['xmlns:content'] = 'http://purl.org/rss/1.0/modules/content/';
    }
    if (isAtom) {
        rss[0]._attr['xmlns:atom'] = 'http://www.w3.org/2005/Atom';
    }
    // rest
    rss[0]._attr['xmlns:cc'] = 'http://web.resource.org/cc/';
    rss[0]._attr['xmlns:itunes'] = 'http://www.itunes.com/dtds/podcast-1.0.dtd';
    rss[0]._attr['xmlns:media'] = 'http://search.yahoo.com/mrss/';
    rss[0]._attr['xmlns:rdf'] = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#';
    return DOCTYPE + xml([{ rss }], true);
};
function invariant(obj, key, msg) {
    if (!obj[key]) {
        const errmsg = key + (msg || ' is missing from your frontmatter');
        console.error(errmsg + '\n ---- \n error found in', obj);
        throw new Error(errmsg);
    }
    return obj[key];
}

// adapted from https://github.com/jpmonette/feed
class Feed {
    constructor(options, IToptions) {
        this.items = [];
        this.categories = [];
        this.contributors = [];
        this.extensions = [];
        this.addItem = (item) => this.items.push(item);
        this.addCategory = (category) => this.categories.push(category);
        this.addContributor = (contributor) => this.contributors.push(contributor);
        this.addExtension = (extension) => this.extensions.push(extension);
        // /**
        //  * Returns a Atom 1.0 feed
        //  */
        // public atom1 = (): string => renderAtom(this)
        /**
         * Returns a RSS 2.0 feed
         */
        this.rss2 = () => renderRSS(this);
        this.options = options;
        this.IToptions = IToptions;
    }
}

const fs = require('fs');
const path = require('path');
const markdownIt = require('markdown-it');
const frontmatter = require('front-matter');
const { parse } = require('date-fns');
const mp3Duration = require('mp3-duration');
//markdownIt is a markdown parser that takes my raw md files and
//translates them into HTML that we can use in the feed
const md = markdownIt({
    html: true,
    linkify: true,
    typographer: true
});
// synchronously grab contents - a separate process because buildFeed needs to be async
const grabContents = (filepaths, myURL) => {
    return filepaths.map(filepath => {
        let { attributes, body } = frontmatter(fs.readFileSync(filepath, 'utf-8'));
        (attributes.slug = filepath.split('.')[0]), // todo: slugify
            // handle local links
            (body = md.render(body));
        body = body.replace(/src="\//g, `src="${myURL}/`);
        const mp3path = path.join(process.cwd(), 'public', attributes.mp3URL);
        return {
            frontmatter: attributes,
            body,
            mp3path,
            filepath
        };
    });
};
// build feed is our main function to build a `Feed` object which we
// can then serialize into various formats
// USER: Customize to your own details
const buildFeed = async (contents, myURL, author, feedOptions, iTunesChannelFields) => {
    let feed = new Feed(feedOptions, iTunesChannelFields);
    feed.addContributor(author);
    await Promise.all(contents.map(async ({ frontmatter: fm, body, mp3path, filepath }) => {
        feed.addItem({
            title: fm.title,
            id: safeJoin(myURL, filepath),
            link: safeJoin(myURL, fm.mp3URL),
            date: parse(fm.date),
            content: body,
            author: [author],
            description: body,
            itunes: {
                // image: // up to you to configure but per-episode image is possible
                duration: await mp3Duration(mp3path, (err) => err && console.error(err.message)),
                // explicit: false, // optional
                // keywords: string[] // per-episode keywords possible
                subtitle: fm.description,
                episodeType: fm.episodeType || 'full',
                episode: fm.episode,
                season: fm.season,
                contentEncoded: body,
                mp3URL: safeJoin(myURL, fm.mp3URL),
                enclosureLength: fs.statSync(mp3path).size // size in bytes
            }
        });
        return {
            frontmatter: fm,
            body
        };
    }));
    return feed;
};
function safeJoin(a, b) {
    /** strip starting/leading slashes and only use our own */
    let a1 = a.slice(-1) === '/' ? a.slice(0, a.length - 1) : a;
    let b1 = b.slice(0) === '/' ? b.slice(1) : b;
    return `${a1}/${b1}`;
}

exports.grabContents = grabContents;
exports.buildFeed = buildFeed;
exports.formatTime = formatTime;
//# sourceMappingURL=podcats.cjs.development.js.map
